use axum::{
    extract::{Path, State},
    middleware,
    routing::post,
    Extension, Json, Router,
};

use crate::{
    db,
    server::{
        error::{AppError, AppResult},
        gpodder::{
            auth_middleware,
            format::{Format, StringWithFormat},
            models::{EpisodeAction, UpdatedUrlsResponse},
        },
        Context,
    },
};

pub fn router(ctx: Context) -> Router<Context> {
    Router::new()
        .route("/{username}", post(post_episode_actions))
        .layer(middleware::from_fn_with_state(ctx.clone(), auth_middleware))
}

async fn post_episode_actions(
    State(ctx): State<Context>,
    Path(username): Path<StringWithFormat>,
    Extension(user): Extension<db::User>,
    Json(actions): Json<Vec<EpisodeAction>>,
) -> AppResult<Json<UpdatedUrlsResponse>> {
    if username.format != Format::Json {
        return Err(AppError::NotFound);
    }

    if *username != user.username {
        return Err(AppError::BadRequest);
    }

    tracing::debug!("{:?}", actions);

    todo!()
}
